'  ---------------------------------------------------------------------------
'  BASIC SPRITE DEMO
'  By Damian Rajkowski (WierdPumpkin)
'  17/12/2004

'  This source code can be freely distributed by any means.

'  THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

'  This is a simple example of sprite animation - a pair of blinking eyes.
'  All the images used in this demo are stored on disk in the .PNG format, which provides an alpha channel
'  (For masking out any parts of the image you don't want) as well as compressing the image without losing
'  any data. PNG is an all-round good format To work in.
'  You'll notice the red background in each image is not shown - its the alpha layer.
'  I use Microsoft Photoeditor To apply alpha. Its easy To use - just point To the color To use as alpha And it will
'  be masked out. You can use any photo-editing software To apply your own alpha.

'  IMPORTANT: Sprite animation has a caveat. Please read carefully where indicated with (*) near the
'  iSpriteTextureSwap function. If you get funny results in your animation Or frames look out of sync,
'  this caveat is probably the reason.

'  After compilation, this source code is turned into a file named 3impactd.dll And saved To the
'  3impactWork folder on your hard disk.

'  ---------------------------------------------------------------------------

Strict
Framework _3Impact.Lib

'  A camera is always required, so that we have a target For rendering.
Global Camera:CAMERA

'  Here's our sprite object - which is actually rendered.
Global Sprite:SPRITE

'  A pair of textures - To animate the eyes - applied To the sprite object.
Global Tex:TEXTURE[3]

'  Constants
Const FRAME__TIME:Float = 5.0      '  Time between frame changes, modify this constant To change the speed of the animation.
Const MOVE__TIME:Float  = 150.0    '  Time between moves, modify this constant To change the speed of repositioning.

'  Variables
Global frame_time:Float = FRAME__TIME      '  Time Until Next frame of animation is displayed. When frame_time reaches zero, show Next frame.
Global frame:Int      = 0               '  Frame of animation To display. Start with first frame (0).
Global increment:Int  = 1               '  Direction To increment frame counter in, either 1 Or -1. Start in positive direction (1).
Global move_time:Float  = MOVE__TIME       '  Time Until Next move. When move_time expires, the face moves To a New position/orientation.

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()

	    '  First, we need a camera Object To render To, Or nothing can be displayed.
	    '  Here we create a Default camera which spans the entire screen.
	    Camera = iCameraCreate(0.0,0.0,1.0,1.0)
	
	    '  Load the sprite Object into our main camera. If you were using multiple cameras you could specify Null as the
	    '  second parameter To allow the sprite To appear in all of them.
	    '  The Sprite Object is now loaded with whatever texture the sprite.x Object file was assigned.
	    '  You could have assigned a texture To sprite.x And Left it at that, And it would render correctly as is.
	    '  But we don't really care what texture the Sprite has because we're replacing it with animation frames.
	    Sprite = iSpriteCreate("basicsprite_res\\sprite.x",Camera)
	
	    '  Scale the sprite To appear slightly larger To start with.
	    iSpriteScaleSet(Sprite,_D3DXVECTOR2(3.0,3.0))
	
	    '  Load textures For animation - this sprite has three frames of animation, load them into the TEXTURE objects
	    '  which aren't themselves displayed, but store data for SPRITE objects to display.
	    Local text:String
	
	    Local i:Int
	
	    For i = 0 To 2
	        '  Use sprintf To prepare a filename with the appropriate suffix.
	        text="basicsprite_res\\frame_"+(i+1)+".png"
	
	        '  Load texture.
	        Tex[i] = iTextureCreate(text)	        
	    Next
	
	    '  Show first frame of animation
	    iSpriteTextureSwap(Sprite,Tex[0])
	
	    '  =========================================================================================================
	    '  (*) IMPORTANT:
	    '  =========================================================================================================
	    '  This may be a little hard To understand at first, but, For performance reasons, there's a peculiarity to
	    '  sprite animation that has To be carefully handled.
	    ' 
	    '  Basically, To animate our sprite we have To change its texture.
	    ' 
	    '  The quickest way To do that is To SWAP the sprite's CURRENT texture with a texture from a TEXTURE object
	    '  (which we have loaded previously - because loading is slow).
	    ' 
	    '  At this point, after the iSpriteTextureSwap command above, our sprite has the texture that used To belong
	    '  To TEXTURE Object Tex[0]; that is, the sprite has the image 'frame_1.png' in it.
	    '  At the same time, Tex[0] has the sprite's original image, which we don't really care about.
	    ' 
	    '  When it comes To displaying the Next frame, we can't simply give Tex[1]'s image to the sprite, because
	    '  Tex[1] will Then receive the sprites image - which came from Tex[0].
	    '  We must SWAP the sprite's image back into Tex[0], and THEN swap the sprite with Tex[1]. And so on.
	    ' 
	    '  Please see the (*) note in the Run() Function below For more information.
	    '  =========================================================================================================
	
	    '  Position sprite at centre-screen.
	    '  NOTE: Sprites operate in a coordinate system where the bottom Left corner of the screen is (-16,-12),
	    '        And the top-Right corner is (16,12). The centre is, logically, (0,0).
	    iSpriteLocationSet(Sprite,_D3DXVECTOR2(0.0,0.0))
	
	    '  Display sprite - do this as the last step.
	    iSpriteShow(Sprite)
	EndMethod

	Method _Run()
   
	    '  Update time by subtracting from it.
	    frame_time :- 1.0
	
	    '  Check If frame_time has expired, And therefore another frame is required.
	    If (frame_time <= 0.0)
	        '  Next frame IS required.
	
	        '  Reset timer To the constant we prepared earlier.
	        frame_time = FRAME__TIME
	
	        '  =========================================================================================================
	        '  (*) IMPORTANT:
	        '  =========================================================================================================
	        '  As explained in the Init() Function, we must swap the sprite's CURRENT texture back into the TEXTURE object
	        '  it came from, before we assign the Next one.
	        '  So here it is - swap the sprite's texture with the Tex[?] it came from.
	        iSpriteTextureSwap(Sprite,Tex[frame])
	
	        '  Increment frame counter in current direction.
	        '  We use the increment variable To allow our animation To loop forwards And backwards. Look at the frame images -
	        '  If we only looped forward And restarted at frame 0 when frame 2 is finished our blinker would look rather strange.
	        '  This way his eye-lids are displayed in the halfway position when opening And closing.
	        frame :+ increment
	
	        '  Check If frame counter has reached either first Or last frame, And reverse direction.
	        If (frame = 0) Then increment = 1 Else If (frame = 2) Then increment = -1
	
	        '  If we're showing the first frame (open eyes), extend the time so that the face isn't blinking constantly.
	        '  We multiply by a random factor To give it a realistic, irregular look.
	        '  Comment out this line If you want To see excessive blinking. :)
	        If (frame = 0) Then frame_time :* iFloatRand(10.0,20.0)
	
	        '  IMPORTANT:
	        '  Show current frame - now that the frame counter is pointing To the Next frame To show, we SWAP that Tex[]
	        '  Object's texture with our sprite. Voila! Animation!
	        iSpriteTextureSwap(Sprite,Tex[frame])
	
	    EndIf
	
	    '  Update moving time.
	    move_time :- 1.0
	
	    '  Check If its time To move.
	    If (move_time <= 0.0) 
	
	        '  Reset move timer.
	        move_time = MOVE__TIME
	
	        '  The following lines give the sprite a New, And random, position/orientation/scale using the
	        '  iFloatRand(from,To) function. Its a useful little func.
	
	        '  Set New sprite position.
	        iSpriteLocationSet(Sprite,_D3DXVECTOR2(iFloatRand(-13.0,13.0),iFloatRand(-9.0,9.0)))
	
	        '  Set New sprite orientation.
	        '  The orientation operates anti-clockwise with 0.0f being To the Right (positive x-axis).
	        iSpriteOrientationSet(Sprite,iFloatRand(0.0,360.0))
	
	        '  Set New sprite size - change the ranges To have some fun.
	        iSpriteScaleSet(Sprite,_D3DXVECTOR2(iFloatRand(1.0,5.0),iFloatRand(1.0,5.0)))
	
	    EndIf
	EndMethod
	
EndType

