' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

'  Cameraman classes provide easy management of dynamic cameras, like chase cameras,
'  on-board, 3rd person/aerial views, And so on. Optionally, you can prevent the camera
'  To go through solid objects, set cameraman quickness, implement user-controlled view
'  mode switching And seamless transitions from one view mode To another.
'  
'  class CameramanLookAtPointKeepDistance_t
'     void Create(CAMERA*,Float,Float,Float,Float,Float,Float,BODY**)
'        Create a cameraman Object which follows the actor And takes it from constant
'        relative distance And altitude.
'        CAMERA* = camera Object To be managed.
'        Float = cameraman body radius. Used For collision detection when BODY**
'                parameter (see below) is Not NULL.
'        Float = relative distance To keep, from actor location
'        Float = relative altitude To keep, from actor location
'        Float = cameraman speed. A good reference value For tweaking is 10.0f.
'                Increasing cameraman speed may cause 'bouncing' around the cameraman's
'                End-location. You can minimize it by increasing damping (Next parameter).
'        Float = cameraman speed damping, between 0.0f (maximum damping) And 1.0f (no damping).
'                A good reference value For tweaking is 0.95f.
'                Note, the smaller the value, the greater the damping effect.
'        Float = camera orientation Delay (approx.), in seconds (0.0f For immediate).
'        BODY** = collision body array. If Not Null, the class will Try preventing the camera
'                 To go through the bodies listed in the specified array.
'                 The array provided can be temporary as the class will use it For
'                 initialization only. The list of bodies must terminate with a Null item.
'     void Update(D3DXVECTOR3*)
'        Call this Function at every dll-loop To tell the cameraman class where the actor is.
'        D3DXVECTOR3* = actor location
'     void Reset()
'        Reset the cameraman To its current End-location And End-orientation.
'     void Resume()
'        Reset the cameraman To the current camera location. Typically used To guarantee
'        a seamless switching from a different cameraman (that is, starting updating
'        this cameraman after stopping updating another cameraman).
'     void Destroy()
'        You must remove the cameraman from memory, before exiting the application, by calling
'        this function.
'  
'  class CameramanLookAtPointKeepOffset_t
'     void Create(CAMERA*,Float,D3DXVECTOR3*,Float,Float,Float,BODY**)
'        Create a cameraman Object which follows the actor And takes it from a constant
'        relative offset, For example from a point located 10 meters away from
'        actor's left side. It is the typical 3rd person view mode.
'        CAMERA* = camera Object To be managed.
'        Float = cameraman body radius. Used For collision detection when BODY**
'                parameter (see below) is Not NULL.
'        D3DXVECTOR3* = offset, relative To actor location And orientation. See Update().
'        Float = cameraman speed. A good reference value For tweaking is 10.0f.
'                Increasing cameraman speed may cause 'bouncing' around the cameraman's
'                End-location. You can minimize it by increasing damping (Next parameter).
'        Float = cameraman speed damping, between 0.0f (maximum damping) And 1.0f (no damping).
'                A good reference value For tweaking is 0.95f.
'                Note, the smaller the value, the greater the damping effect.
'        Float = camera orientation Delay (approx.), in seconds (0.0f For immediate).
'        BODY** = collision body array. If Not Null, the class will Try preventing the camera
'                 To go through the bodies listed in the specified array.
'                 The array provided can be temporary as the class will use it For
'                 initialization only. The list of bodies must terminate with a Null item.
'     void Update(D3DXVECTOR3*,D3DXQUATERNION*)
'        Call this Function at every dll-loop To tell the cameraman class where the actor is
'        And how it is oriented.
'        D3DXVECTOR3* = actor location
'        D3DXQUATERNION* =  actor orientation
'     void Reset()
'        Reset the cameraman To its current End-location And End-orientation.
'     void Resume()
'        Reset the cameraman To the current camera location. Typically used To guarantee
'        a seamless switching from a different cameraman (that is, starting updating
'        this cameraman after stopping updating another cameraman).
'     void Destroy()
'        You must remove the cameraman from memory, before exiting the application, by calling
'        this function.
'  
'  class CameramanFixed_t
'     Note: To define, use the following syntax:
'     CameramanFixed_t MyCameraman(number-of-fixed-cameras)
'     void Create(CAMERA*,Float,BODY**)
'        Create a cameraman Object which takes the actor from a set of fixed locations.
'        CAMERA* = camera Object To be managed.
'        Float = cameraman body radius. Used For collision detection when BODY**
'                parameter (see below) is Not NULL.
'        BODY** = collision body array. If Not Null, the class will Try preventing the camera
'                 To go through the bodies listed in the specified array, by temporarily
'                 displacing it when hit.
'                 The array provided can be temporary as the class will use it For
'                 initialization only. The list of bodies must terminate with a Null item.
'     void Update(D3DXVECTOR3*)
'        Call this Function at every dll-loop To tell the cameraman class where the actor is.
'        You must create at least one fixed location with LocationAdd() before calling
'        this function.
'        D3DXVECTOR3* = actor location
'     void LocationAdd(D3DXVECTOR3*,Float,Float,Float)
'        Add a fixed location To the set of available locations. The cameraman will take the
'        actor from the location which is closest To it.
'        D3DXVECTOR3* location
'        Float = cameraman speed, on returning To its natural location, after a collision has
'                displaced it. A good reference value For tweaking is 5.0f.
'                Increasing cameraman speed may cause 'bouncing' around the cameraman's
'                natural location. You can minimize it by increasing damping (Next parameter).
'        Float = cameraman speed damping, between 0.0f (maximum damping) And 1.0f (no damping).
'                A good reference value For tweaking is 0.95f.
'                Note, the smaller the value, the greater the damping effect.
'        Float = camera orientation Delay (approx.), in seconds (0.0f For immediate).
'     void Reset()
'        Reset cameraman orientation To exactly take the actor.
'     void Resume()
'        Reset the cameraman To the current camera location. Typically used To guarantee
'        a seamless switching from a different cameraman (that is, starting updating
'        this cameraman after stopping updating another cameraman).
'     void Destroy()
'        You must remove the cameraman And all added locations from memory, before exiting
'        the application, by calling this function.
'  
'  class CameramanOnBody_t
'     void Create(CAMERA*,BODY*,D3DXVECTOR3*,D3DXVECTOR3*,Float,Float)
'        Create a cameraman Object which sticks To a specified body. Typically used
'        To implement 1st person view modes.
'        CAMERA* = camera Object To be managed.
'        BODY* = body Object To attach the cameraman To
'        D3DXVECTOR3* = source-model-center relative location To attach the cameraman to.
'        D3DXVECTOR3* = source-model-center relative location To look at
'        Float = camera translation Delay (approx.), in seconds (0.0f For immediate).
'        Float = camera orientation Delay (approx.), in seconds (0.0f For immediate).
'     void Update()
'        Call this Function at every dll-loop To keep the cameraman attached To the body.
'     void Reset()
'        Reset the cameraman To its on-body natural location And orientation.
'     void Resume()
'        Reset the cameraman To the current camera location. Typically used To guarantee
'        a seamless switching from a different cameraman (that is, starting updating
'        this cameraman after stopping updating another cameraman).
'     void Destroy()
'        You must remove the cameraman from memory, before exiting the application, by calling
'        this function.


Type CameramanLookAtPointKeepDistance_t

   Field cCamera:CAMERA
   Field cReset:Int
   Field cCameramanBody:BODY
   Field cCameramanSpring:SPRING
   Field cDistance:Float
   Field cAltitude:Float
   Field cSpring:Float
   Field cDamping:Float
   Field cOrientationTime:Float
   Field cBodyBody:BODYBODY[]
   Field cBodyBodyCount:Int

   Method New()
      cCamera = Null
      cReset = True
      cCameramanBody = Null
      cCameramanSpring = Null
      cBodyBody = Null
   EndMethod

   Method create(camera:CAMERA,radius:Float,distance:Float,altitude:Float,spring:Float,damping:Float,orientationtime:Float ,scenerybody:BODY[])
      cCamera = camera
      cDistance = distance
      cAltitude = altitude
      cSpring = spring
      cDamping = damping
      cOrientationTime = orientationtime

      ' create cameraman body And spring
      cCameramanBody=iBodySphereCreate(radius,_D3DXVECTOR3(0.0,0.0,0.0),1.0)
      iBodyDampingSet(cCameramanBody,0.0,cDamping,0.0,0.95)
      Local v:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,v)
      iBodyLocationSet(cCameramanBody,v,True)
      cCameramanSpring = iSpringCreate(cCameramanBody,_D3DXVECTOR3(0.0,0.0,0.0),Null,v)
      iSpringSet(cCameramanSpring,cSpring)

      If (scenerybody)
         cBodyBodyCount = 0
         While (cBodyBodyCount<scenerybody.length And scenerybody[cBodyBodyCount])
            cBodyBodyCount:+1
         Wend
         cBodyBody = New BODYBODY[cBodyBodyCount]
         For Local i:Int=0 To cBodyBodyCount-1
            cBodyBody[i] = iBodyBodyCreate(cCameramanBody,scenerybody[i])
            iBodyBodyRestitutionSet(cBodyBody[i],0.0)
            iBodyBodyFrictionSet(cBodyBody[i],0.0)
         Next
      EndIf

      iBodyEnable(cCameramanBody)
      iSpringEnable(cCameramanSpring)
   EndMethod

   Method Update(actorlocation:D3DXVECTOR3)
      Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local cameramanlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local cameramanorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local endlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local endorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local v:D3DXVECTOR3=_D3DXVECTOR3()

      ' store some data we need later
      iCameraOrientation(cCamera,cameraorientation)
      iBodyLocation(cCameramanBody,cameramanlocation)

      ' compute End location For the cameraman, a location that satisfies
      ' the required distance And altitude from the actor...
      iVectorLengthSet(v,(cameramanlocation.Minus(actorlocation)),cDistance)
      endlocation = actorlocation.Plus(v)
      endlocation.y = actorlocation.y+cAltitude
      ' ...And set it as spring target, the place where the cameraman is attracted To
      iSpringTargetBodySet(cCameramanSpring,Null,endlocation)

      ' convert orientation time factor To a proper, dll-loop-frequency-dependant, format
      Local orientationtime:Float = cOrientationTime
      If (orientationtime>iFrq())
		orientationtime = iFrq()/orientationtime
      Else
		orientationtime=1.0
	 EndIf

      If (cReset)
         cReset = False
         iQuaternionLookAt(endorientation,(actorlocation.Minus(endlocation)),Null)
         iBodyOrientationSet(cCameramanBody,endorientation,True)
         iBodyLocationSet(cCameramanBody,endlocation,True)
         iCameraOrientationSet(cCamera,endorientation)
         iCameraLocationSet(cCamera,endlocation)
      Else
         ' compute End orientation required To look at the actor
         ' from current cameraman location
         iQuaternionLookAt(endorientation,(actorlocation.Minus(cameramanlocation)),Null)
         ' we want the cameraman To smoothly tend To the ideal orientation, over time.
         ' So, compute an interpolated orientation (by user defined factor)
         iQuaternionInterpolate(cameramanorientation,cameraorientation,endorientation,orientationtime) 
         ' apply current cameraman orientation And location To the actual camera Object
         iCameraOrientationSet(cCamera,cameramanorientation)
         iCameraLocationSet(cCamera,cameramanlocation)
      EndIf
   EndMethod
   Method Reset()
      cReset = True
   EndMethod
   Method Resume()
      Local cameralocation:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,cameralocation)
      iBodyLocationSet(cCameramanBody,cameralocation,True)
      cReset = False
   EndMethod
   Method Destroy()
      If (cBodyBody)
         For Local i:Int=0 To cBodyBodyCount-1
            cBodyBody[i]=Null
         Next
         cBodyBody=cBodyBody[..0]
         cBodyBody = Null
      EndIf
      cCameramanBody=Null
   EndMethod

   Method Delete()
	 Destroy()
   EndMethod
EndType

Type CameramanLookAtPointKeepOffset_t

   Field cReset:Int
   Field cCamera:CAMERA
   Field cCameramanBody:BODY
   Field cCameramanSpring:SPRING
   Field cOffset:D3DXVECTOR3=_D3DXVECTOR3()
   Field cSpring:Float
   Field cDamping:Float
   Field cOrientationTime:Float
   Field cBodyBody:BODYBODY[]
   Field cBodyBodyCount:Int

   Method New()
      cReset = True
      cCamera = Null
      cCameramanBody = Null
      cCameramanSpring = Null
      cBodyBody = Null
   EndMethod

   Method create(camera:CAMERA,radius:Float,offset:D3DXVECTOR3,spring:Float,damping:Float,orientationtime:Float,scenerybody:BODY[])
      cCamera = camera
      cOffset = offset
      cSpring = spring
      cDamping = damping
      cOrientationTime = orientationtime

      ' create cameraman body And spring
      cCameramanBody=iBodySphereCreate(radius,_D3DXVECTOR3(0.0,0.0,0.0),1.0)
      iBodyDampingSet(cCameramanBody,0.0,cDamping,0.0,0.95)
      Local v:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,v)
      iBodyLocationSet(cCameramanBody,v,True)
      cCameramanSpring = iSpringCreate(cCameramanBody,_D3DXVECTOR3(0.0,0.0,0.0),Null,v)
      iSpringSet(cCameramanSpring,cSpring)

      If (scenerybody)
         cBodyBodyCount = 0
         While (cBodyBodyCount<scenerybody.length And scenerybody[cBodyBodyCount])
            cBodyBodyCount:+1
         Wend
         cBodyBody = New BODYBODY[cBodyBodyCount]
         For Local i:Int=0 To cBodyBodyCount-1
            cBodyBody[i] = iBodyBodyCreate(cCameramanBody,scenerybody[i])
            iBodyBodyRestitutionSet(cBodyBody[i],0.0)
            iBodyBodyFrictionSet(cBodyBody[i],0.0)
         Next
      EndIf

      iBodyEnable(cCameramanBody)
      iSpringEnable(cCameramanSpring)
   EndMethod
   Method Update(actorlocation:D3DXVECTOR3,actororientation:D3DXQUATERNION)
      Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local cameramanlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local cameramanorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local endlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local endorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local v:D3DXVECTOR3=_D3DXVECTOR3()

      ' store some data we need later
      iCameraOrientation(cCamera,cameraorientation)
      iBodyLocation(cCameramanBody,cameramanlocation)

      ' compute End location For the cameraman, a location that satisfies
      ' the required offset from the actor And set it as spring target, the
      ' place where the cameraman is attracted To

      ' find actor's local Z axis, as projected onto absolute XZ plane
      iVectorRotate(v,_D3DXVECTOR3(0.0,0.0,1.0),actororientation)
      v.y = 0.0
      ' find angle rotating absolute Z over Local Z
      Local angle:Float=iVectorsAngle(v,_D3DXVECTOR3(0.0,0.0,1.0))
      If (v.x < 0.0) angle = -angle
      ' re-orient offset vector about Y axis To properly follow actor's Y orientation
      iVectorEulerRotate(v,cOffset,0.0,angle,0.0,"xyz")
      ' End location is computed absolute offset + current actor location
      endlocation = v.Plus(actorlocation)
      ' set as spring target
      iSpringTargetBodySet(cCameramanSpring,Null,endlocation)

      ' convert orientation time factor To a proper, dll-loop-frequency-dependant, format
      Local orientationtime:Float = cOrientationTime
      If (orientationtime>iFrq()) 
		orientationtime = iFrq()/orientationtime
      Else 
		orientationtime=1.0
	 EndIf

      If (cReset)
         cReset = False
         iQuaternionLookAt(endorientation,(actorlocation.Minus(endlocation)),Null)
         iBodyOrientationSet(cCameramanBody,endorientation,True)
         iBodyLocationSet(cCameramanBody,endlocation,True)
         iCameraOrientationSet(cCamera,endorientation)
         iCameraLocationSet(cCamera,endlocation)
      Else
         ' compute End orientation required To look at the actor
         ' from current cameraman location
         iQuaternionLookAt(endorientation,(actorlocation.Minus(cameramanlocation)),Null)
         ' we want the cameraman To smoothly tend To the ideal orientation, over time.
         ' So, compute an interpolated orientation (by user defined factor)
         iQuaternionInterpolate(cameramanorientation,cameraorientation,endorientation,orientationtime) 
         ' apply current cameraman orientation And location To the actual camera Object
         iCameraOrientationSet(cCamera,cameramanorientation)
         iCameraLocationSet(cCamera,cameramanlocation)
      EndIf
   EndMethod
   Method Reset()
      cReset = True
   EndMethod
   Method Resume()
      Local cameralocation:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,cameralocation)
      iBodyLocationSet(cCameramanBody,cameralocation,True)
      cReset = False
   EndMethod
   Method Destroy()
      If (cBodyBody)
         For Local i:Int=0 To cBodyBodyCount-1
            cBodyBody[i]=Null
         Next
         cBodyBody=cBodyBody[..0]
         cBodyBody = Null
      EndIf
      cCameramanBody=Null
   EndMethod
   Method Delete()
	 Destroy()
   EndMethod
EndType
Type FIXEDCAMERAMAN
   Field location:D3DXVECTOR3=_D3DXVECTOR3()
   Field spring:Float
   Field damping:Float
   Field orientationtime:Float
EndType

Type CameramanFixed_t

   Field cFixedCameraman:FIXEDCAMERAMAN[]
   Field cFixedCameramanCount:Int
   Field cFixedCameramanMax:Int
   Field cOldLocationId:Int
   Field cCamera:CAMERA
   Field cCameramanBody:BODY
   Field cCameramanSpring:SPRING
   Field cBodyBody:BODYBODY[]
   Field cBodyBodyCount:Int
   Field cResume:Int

   Method New()
      cCamera = Null
      cCameramanBody = Null
      cCameramanSpring = Null
      cOldLocationId = $ffffffff
      cBodyBody = Null
   EndMethod

   Method initCameramanFixed_t(count:Int)
	 cFixedCameraman=cFixedCameraman[..0]
	 cFixedCameraman=Null
      cFixedCameramanMax = count
      cFixedCameraman = New FIXEDCAMERAMAN[cFixedCameramanMax]
   EndMethod

   Method Delete()
      cFixedCameraman=cFixedCameraman[..0]
	 cFixedCameraman=Null
	 Destroy()
   EndMethod

   Method create(camera:CAMERA,radius:Float,scenerybody:BODY[])

      cCamera = camera

      ' create cameraman body And spring
      cCameramanBody=iBodySphereCreate(radius,_D3DXVECTOR3(0.0,0.0,0.0),1.0)
      iBodyDampingSet(cCameramanBody,0.0,0.95,0.0,0.95)
      Local v:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,v)
      iBodyLocationSet(cCameramanBody,v,True)
      cCameramanSpring = iSpringCreate(cCameramanBody,_D3DXVECTOR3(0.0,0.0,0.0),Null,v)
      iSpringSet(cCameramanSpring,10.0)

      If (scenerybody)
         cBodyBodyCount = 0
         While (cBodyBodyCount<scenerybody.length And scenerybody[cBodyBodyCount])
            cBodyBodyCount:+1
         Wend
         cBodyBody = New BODYBODY[cBodyBodyCount]
         For Local i:Int=0 To cBodyBodyCount-1
            cBodyBody[i] = iBodyBodyCreate(cCameramanBody,scenerybody[i])
            iBodyBodyRestitutionSet(cBodyBody[i],0.0)
            iBodyBodyFrictionSet(cBodyBody[i],0.0)
         Next
      EndIf

      iBodyEnable(cCameramanBody)
      iSpringEnable(cCameramanSpring)

      cResume = False
   EndMethod
   Method Update(actorlocation:D3DXVECTOR3)
      Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local cameramanlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local endlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local endorientation:D3DXQUATERNION=_D3DXQUATERNION()

      Local i:Int
      Local locationid:Int
      Local distance:Float,mindistance:Float

      If Not cFixedCameramanCount Return

      ' find closest location
      mindistance=FLT_MAX
      For i=0 To cFixedCameramanCount-1
         distance=iVectorLengthSq(cFixedCameraman[i].location.Minus(actorlocation))
         If (distance<mindistance)
            mindistance=distance
            locationid=i
         EndIf
      Next
      endlocation = cFixedCameraman[locationid].location

      ' store some data we need later
      iCameraOrientation(cCamera,cameraorientation)
      iBodyLocation(cCameramanBody,cameramanlocation)

      ' set End location as spring target, the place where the cameraman is attracted To
      iSpringTargetBodySet(cCameramanSpring,Null,endlocation)

      If ((locationid <> cOldLocationId) And Not cResume)
         ' a resume was Not called For And
         ' we have just switched camera, so force the camera To
         ' the New location And exactly take the actor
         iBodyLocationSet(cCameramanBody,endlocation,True)
         iCameraLocationSet(cCamera,endlocation)
         iCameraLookAt(cCamera,actorlocation,1.0)
         ' also, set other specific parameters For the New location
         iSpringSet(cCameramanSpring,cFixedCameraman[locationid].spring)
         iBodyDampingSet(cCameramanBody,0.0,cFixedCameraman[locationid].damping,0.0,0.95)
      Else
         ' convert orientation time factor To a proper, dll-loop-frequency-dependant, format
         Local orientationtime:Float = cFixedCameraman[locationid].orientationtime
         If (orientationtime>iFrq())
			orientationtime = iFrq()/orientationtime
         Else 
			orientationtime=1.0
	    EndIf

         ' compute End orientation required To look at the actor
         ' from current cameraman location
         iQuaternionLookAt(endorientation,actorlocation.Minus(cameramanlocation),Null)
         ' we want the cameraman To smoothly tend To the ideal orientation, over time.
         ' So, change endorientation To an interpolated orientation (by user defined factor)
         iQuaternionInterpolate(endorientation,cameraorientation,endorientation,orientationtime) 
         ' apply current cameraman orientation And location To the actual camera Object
         iCameraOrientationSet(cCamera,endorientation)
         iCameraLocationSet(cCamera,cameramanlocation)
      EndIf
      cOldLocationId=locationid
      cResume = False
   EndMethod
   Method Reset()
      cOldLocationId = $ffffffff
      cResume = False
   EndMethod
   Method Resume()
      Local cameralocation:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,cameralocation)
      iBodyLocationSet(cCameramanBody,cameralocation,True)
      cResume = True
   EndMethod
   Method LocationAdd(location:D3DXVECTOR3,spring:Float,damping:Float,orientationtime:Float)
      If (cFixedCameramanCount >= cFixedCameramanMax)
         Return
      EndIf
      ' set data For the New camera
      Local i:Int = cFixedCameramanCount
      cFixedCameraman[i].location=location
      cFixedCameraman[i].spring=spring
      cFixedCameraman[i].damping=damping
      cFixedCameraman[i].orientationtime=orientationtime
      cFixedCameramanCount:+1
   EndMethod
   Method Destroy()
      If (cBodyBody)
         For Local i:Int=0 To cBodyBodyCount-1
            cBodyBody[i]=Null
         Next
         cBodyBody=cBodyBody[..0]
         cBodyBody = Null
      EndIf
      cCameramanBody=Null
   EndMethod
EndType
Type CameramanOnBody_t

   Field cCamera:CAMERA
   Field cBody:BODY
   Field cReset:Int
   Field cLocation:D3DXVECTOR3
   Field cLookAt:D3DXVECTOR3
   Field cTranslationTime:Float
   Field cOrientationTime:Float

   Method New()
      cCamera = Null
      cBody = Null
   EndMethod
   Method create(camera:CAMERA,body:BODY,location:D3DXVECTOR3,lookat:D3DXVECTOR3,translationtime:Float,orientationtime:Float)
      cCamera = camera
      cReset = True
      cBody = body
      cLocation = location
      cLookAt = lookat
      cTranslationTime = translationtime
      cOrientationTime = orientationtime
   EndMethod
   Method Update()
   
      Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local cameralocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local cameramanlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local cameramanorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local lookat:D3DXVECTOR3=_D3DXVECTOR3()
      Local endlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local endorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local tilt:D3DXVECTOR3=_D3DXVECTOR3()

      ' store some data we need later
      iCameraOrientation(cCamera,cameraorientation)
      iCameraLocation(cCamera,cameralocation)

      ' find world relative locations For cameraman And look at point
      iVectorBodyToWorldTransform(endlocation,cLocation,cBody,0)
      iVectorBodyToWorldTransform(lookat,cLookAt,cBody,0)
      ' find absolute orientation For the up vector
      iVectorBodyToWorldRotate(tilt,_D3DXVECTOR3(0.0,1.0,0.0),cBody,0)

      ' convert translation time factor To a proper, dll-loop-frequency-dependant, format
      Local translationtime:Float = cTranslationTime
      If (translationtime>iFrq())
		 translationtime = iFrq()/translationtime
      Else
		 translationtime=1.0
	 EndIf

      ' compute an interpolated position, by user-defined factor
      cameramanlocation = cameralocation.Plus(endlocation.Minus(cameralocation).Mult(translationtime))

      ' convert orientation time factor To a proper, dll-loop-frequency-dependant, format
      Local orientationtime:Float = cOrientationTime
      If (orientationtime>iFrq())
		 orientationtime = iFrq()/orientationtime
      Else
		 orientationtime=1.0
	 EndIf
	
      ' compute End orientation required To look at the actor
      ' from current cameraman location
      iQuaternionLookAt(endorientation,lookat.Minus(endlocation),tilt)
      ' we want the cameraman To smoothly tend To the ideal orientation, over time.
      ' So, compute an interpolated orientation (by user defined factor)
      iQuaternionInterpolate(cameramanorientation,cameraorientation,endorientation,orientationtime) 

      If (cReset)
         cReset = False
         iCameraOrientationSet(cCamera,endorientation)
         iCameraLocationSet(cCamera,endlocation)
      Else
         ' apply current cameraman orientation And location To the actual camera Object
         iCameraOrientationSet(cCamera,cameramanorientation)
         iCameraLocationSet(cCamera,cameramanlocation)
      EndIf
   EndMethod
   Method Reset()
      cReset = True
   EndMethod
   Method Resume()
      cReset = False
   EndMethod
   Method Destroy()
      ' nothing was created in current implementation
   EndMethod
EndType

Type CameramanLookAtPointKeepDistanceFast_t

   ' Usage is identical To CameramanLookAtPointKeepDistance_t
   ' but the cameraman is fast enough here To follow any
   ' actor, no matter how fast it moves.

   Field cCamera:CAMERA
   Field cReset:Int
   Field cCameramanBody:BODY
   Field cDistance:Float
   Field cAltitude:Float
   Field cSpring:Float
   Field cDamping:Float
   Field cOrientationTime:Float 
   Field cBodyBody:BODYBODY[]
   Field cBodyBodyCount:Int 

   Method New()
      cCamera = Null
      cReset = True
      cCameramanBody = Null
      cBodyBody = Null
   EndMethod
   Method create(camera:CAMERA,radius:Float,distance:Float,altitude:Float,spring:Float,damping:Float,orientationtime:Float,scenerybody:BODY[])

      cCamera = camera
      cDistance = distance
      cAltitude = altitude
      cSpring = spring
      cDamping = damping
      cOrientationTime = orientationtime

      ' create cameraman body
      cCameramanBody=iBodySphereCreate(radius,_D3DXVECTOR3(0.0,0.0,0.0),1.0)
      iBodyDampingSet(cCameramanBody,0.0,cDamping,0.0,0.95)
      Local v:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,v)
      iBodyLocationSet(cCameramanBody,v,True)

      If (scenerybody)
         cBodyBodyCount = 0
         While (cBodyBodyCount<scenerybody.length And scenerybody[cBodyBodyCount])
            cBodyBodyCount:+1
         Wend
         cBodyBody = New BODYBODY[cBodyBodyCount]
         For Local i:Int=0 To cBodyBodyCount-1
            cBodyBody[i] = iBodyBodyCreate(cCameramanBody,scenerybody[i])
            iBodyBodyRestitutionSet(cBodyBody[i],0.0)
            iBodyBodyFrictionSet(cBodyBody[i],0.0)
         Next
      EndIf

      iBodyEnable(cCameramanBody)
   EndMethod
   Method Update(actorlocation:D3DXVECTOR3)
   
      Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local cameramanlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local cameramanorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local endlocation:D3DXVECTOR3=_D3DXVECTOR3()
      Local endorientation:D3DXQUATERNION=_D3DXQUATERNION()
      Local v:D3DXVECTOR3=_D3DXVECTOR3()

      ' store some data we need later
      iCameraOrientation(cCamera,cameraorientation)
      iBodyLocation(cCameramanBody,cameramanlocation)

      ' compute End location For the cameraman, a location that satisfies
      ' the required distance And altitude from the actor...
      iVectorLengthSet(v,cameramanlocation.Minus(actorlocation),cDistance)
      endlocation = actorlocation.Plus(v)
      endlocation.y = actorlocation.y+cAltitude
      ' ...And from it build a force that pushes the camerameraman To it.
      Local l:Float = iVectorLength(cameramanlocation.Minus(endlocation))
      iVectorLengthSet(v,endlocation.Minus(cameramanlocation),l*cSpring)
      Local d:Float = iFloatInterpolate(iVectorLength(cameramanlocation.Minus(endlocation)),1.0,1000.0,cDamping,1.0,True)
      iBodyDampingSet(cCameramanBody,0.0,d,0.0,0.95)
      iBodyAccelerationApply(cCameramanBody,v)

      ' convert orientation time factor To a proper, dll-loop-frequency-dependant, format
      Local orientationtime:Float = cOrientationTime
      If (orientationtime>iFrq())
		 orientationtime = iFrq()/orientationtime
      Else
		 orientationtime=1.0
	 EndIf

      If (cReset)
         cReset = False
         iQuaternionLookAt(endorientation,actorlocation.Minus(endlocation),Null)
         iBodyOrientationSet(cCameramanBody,endorientation,True)
         iBodyLocationSet(cCameramanBody,endlocation,True)
         iCameraOrientationSet(cCamera,endorientation)
         iCameraLocationSet(cCamera,endlocation)
      Else
         ' compute End orientation required To look at the actor
         ' from current cameraman location
         iQuaternionLookAt(endorientation,actorlocation.Minus(cameramanlocation),Null)
         ' we want the cameraman To smoothly tend To the ideal orientation, over time.
         ' So, compute an interpolated orientation (by user defined factor)
         iQuaternionInterpolate(cameramanorientation,cameraorientation,endorientation,orientationtime) 
         ' apply current cameraman orientation And location To the actual camera Object
         iCameraOrientationSet(cCamera,cameramanorientation)
         iCameraLocationSet(cCamera,cameramanlocation)
      EndIf
   EndMethod
   Method Reset()
      cReset = True
   EndMethod
   Method Resume()
      Local cameralocation:D3DXVECTOR3=_D3DXVECTOR3()
      iCameraLocation(cCamera,cameralocation)
      iBodyLocationSet(cCameramanBody,cameralocation,True)
      cReset = False
   EndMethod
   Method Destroy()
      If (cBodyBody)
         For Local i:Int=0 To cBodyBodyCount-1
            cBodyBody[i]=Null
         Next
         cBodyBody=cBodyBody[..0]
         cBodyBody = Null
      EndIf
      cCameramanBody=Null
   EndMethod
   Method Delete()
	 Destroy()
   EndMethod
EndType

