' Environment dynamic map demo
' Adds some code To the simple environment map demo To demonstrate environment
' mapping that reflects the actual environment.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Craft:BODY
Global CraftMesh:MESH
Global CraftMeshShadow:MESH
Global CraftTerrain:BODYBODY
Global Envmap:ENVMAP

' We need two more meshes For this example
Global BallMesh:MESH
Global BallMeshShadow:MESH

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Craft=iBodySGCreate("default_res\\craft_.spg",0.13)
	   CraftMesh=iBodyMeshCreate("default_res\\craft.x",Craft)
	   iBodyLocationSet(Craft,_D3DXVECTOR3(0.0,10.0,0.0),True)
	   iBodyDampingSet(Craft,0.0,0.999,0.0,0.995)
	   CraftMeshShadow=iMeshBodyShadowCasterCreate("default_res\\craft_shadow.x",Craft,500.0)
	
	   CraftTerrain=iBodyBodyCreate(Craft,Terrain)
	   iBodyBodyFrictionSet(CraftTerrain,2.0)
	
	   Envmap=iEnvMapCreate("default_res\\envmap.dds")
	   iMeshEnvMapReceiverAdd(CraftMesh,Envmap)
	   iMeshEnvMapBlendSet(CraftMesh,0.85)
	
	   ' Creates a ball mesh with shadow (no collisions) To better
	   ' show that real-time reflection works
	   BallMesh=iMeshCreate("default_res\\ball.x")
	   iMeshLocationSet(BallMesh,_D3DXVECTOR3(2.0,3.0,-1.0))
	   BallMeshShadow=iMeshShadowCasterCreate("default_res\\ball_shadow.x",10.0)
	   iMeshLocationSet(BallMeshShadow,_D3DXVECTOR3(2.0,3.0,-1.0))
	
	   ' Dynamic environment mapping basically updates the reflected image
	   ' so that it represents the actual surrounding, as seen by a specified
	   ' viewpoint. It is achieved by rendering the scene To a texture.
	   ' Basically the engines, before rendering the scene To the screen,
	   ' renders it To the six images that form the cube texture, which is the
	   ' reflected image.
	
	   ' Cube texture rendering is a GPU/CPU intensive task, but you can
	   ' adjust the number of renderings per second, thus defining the
	   ' degree of synchronization between the actual environment And the
	   ' reflected image of it, by calling the iEnvMapUpdateRateSet() function.
	
	   ' The following line sets the number of texture renderings per second.
	   ' The maximum is 6.0f/iFrq(), that is 450.0, a lot of GPU work!
	   iEnvMapUpdateRateSet(Envmap,50.0)
	
	   ' Tells the engine what meshes should be reflected.
	   ' Note that, If a skybox is visible in the scene,
	   ' you must always specify a skybox among them!
	   iSkyBoxEnvMapCasterAdd(SkyBox)
	   iMeshEnvMapCasterAdd(TerrainMesh)
	   iMeshEnvMapCasterAdd(BallMesh)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Craft)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(CraftMesh)
	   iMeshShow(CraftMeshShadow)
	   ' Don't forget to show the two new meshes...
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	EndMethod
	
	Method _Run()
	
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Craft,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Craft,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		iBodyAccelerationApply(Craft,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else 
		iBodyAccelerationApply(Craft,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   iWindowTitleSet(iFrameRate(True))
	
	   ' Because the environment mapped mesh (Craft) is moving, we must update
	   ' the viewpoint used by the engine To render the environment To the cube
	   ' texture. The environment dynamic map viewpoint is called 'focus'.
	   iEnvMapFocusSet(Envmap,cameratarget)
	EndMethod
EndType

