' Environment map demo

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Craft:BODY
Global CraftMesh:MESH
Global CraftMeshShadow:MESH
Global CraftTerrain:BODYBODY

' Defines the Object variable For the environment map
Global Envmap:ENVMAP

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Craft=iBodySGCreate("default_res\\craft_.spg",0.13)
	   CraftMesh=iBodyMeshCreate("default_res\\craft.x",Craft)
	   iBodyLocationSet(Craft,_D3DXVECTOR3(0.0,10.0,0.0),True)
	   iBodyDampingSet(Craft,0.0,0.999,0.0,0.995)
	   CraftMeshShadow=iMeshBodyShadowCasterCreate("default_res\\craft_shadow.x",Craft,500.0)
	
	   CraftTerrain=iBodyBodyCreate(Craft,Terrain)
	   iBodyBodyFrictionSet(CraftTerrain,2.0)
	
	   ' Environment maps are handled as 'objects'. You can consider the environment
	   ' map Object as the reflected image of the surroundings. That is, the 'object'
	   ' is the 'reflected-image', and it can be assigned to any mesh, so that the mesh
	   ' 'reflects' the environment image, producing a convincing reflection effect.
	
	   ' The environment map Object is created from a cube-texture stored in a .dds file.
	   ' The following line creates the environment map Object from a cube-texture file
	   ' named 'envmap.dds' and puts its 'handle' into an ENVMAP* object variable named Envmap.
	   Envmap=iEnvMapCreate("default_res\\envmap.dds")
	   ' The environment map Object can Then be assigned To any mesh. As a result, the
	   ' mesh will reflect the environment image. The following line assigns the Envmap
	   ' To the CraftMesh.
	   iMeshEnvMapReceiverAdd(CraftMesh,Envmap)
	   ' Each mesh can reflect the environment map with different 'intensity'. The intensity
	   ' factor can be anywhere from 0.0f (no reflection) To 1.0f (maximum reflection).
	   ' The following line sets the environment reflection intensity For the CraftMesh.
	   iMeshEnvMapBlendSet(CraftMesh,0.25)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Craft)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(CraftMesh)
	   iMeshShow(CraftMeshShadow)
	EndMethod
	
	Method _Run()
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Craft,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Craft,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		iBodyAccelerationApply(Craft,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		iBodyAccelerationApply(Craft,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   iWindowTitleSet(iFrameRate(True))
	EndMethod
EndType
