' Gamma-effects demo.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY

Global FontWhite:FONT

Global MessageString:String

Global GammaMultiply:Float
Global GammaAdd:Float

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	
	   ' Initialize the gamma variables
	   GammaMultiply = 1.0
	   GammaAdd = 0.0
	EndMethod
	Method _Run()
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   ' First of all, memorize the sum of gamma bias And offset factors.
	   Local gammasum:Float = GammaMultiply+GammaAdd
	   ' Adjust gamma factors on key presses
	   If (iKeyDown(DIK_LSHIFT) Or iKeyDown(DIK_RSHIFT))
	      ' [Shift] is pressed. The user wants To adjust gamma bias.
	      ' Increase bias If [A] is pressed, decrease it If [Z] is pressed
	      If (iKeyClick(DIK_A)) GammaMultiply :+ 0.05
	      If (iKeyClick(DIK_Z )) GammaMultiply :- 0.05
	      ' Make sure bias is within acceptable values
	      If (GammaMultiply > 10.0) GammaMultiply = 10.0
	      If (GammaMultiply < 0.0) GammaMultiply = 0.0
	   Else
	      ' [Shift] is Not pressed. The user wants To adjust gamma offset.
	      ' Increase offset If [A] is pressed, decrease it If [Z] is pressed
	      If (iKeyClick(DIK_A)) GammaAdd :+ 0.05
	      If (iKeyClick(DIK_Z )) GammaAdd :- 0.05
	      ' Make sure offset is within acceptable values
	      If (GammaAdd > 1.0) GammaAdd = 1.0
	      If (GammaAdd < -1.0) GammaAdd = -1.0
	   EndIf
	   ' If the sum of gamma bias And offset has changed, it is time To call
	   ' iGammaSet() To apply the changes. This ensures we call the Function
	   ' when it is necessary only. It is a computationally expensive Function
	   ' And you should Not call it at every DLL loop.
	   If (gammasum <> GammaMultiply+GammaAdd) iGammaSet(GammaMultiply,GammaMultiply,GammaMultiply,GammaAdd,GammaAdd,GammaAdd)
	
	   Local temp:String
	   temp=GammaAdd
	   MessageString="GammaAdd: "+temp[0..6]
	   temp=GammaMultiply	
	   MessageString:+" - GammaMultiply: "+temp[0..6]
	   iPrintCentered(MessageString,_D3DXVECTOR2(0.0,10.0),_D3DXVECTOR2(1.0,1.0),0.1,FontWhite,0.0)
	   iPrintCentered("Press [A]/[Z] to simulate fading, [Shift]+[A]/[Z] to adjust gamma",_D3DXVECTOR2(0.0,9.0),_D3DXVECTOR2(0.5,0.5),0.1,FontWhite,0.0)

	   iWindowTitleSet(iFrameRate(True))			
	EndMethod
EndType




