' Glass rendering effect demo
' A modified version of the EnvironmentDynamicMap demo To
' demonstrate real-time refraction of the actual environment.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY
Global Envmap:ENVMAP

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	   iShadowColorSet(_D3DXVECTOR3(0.95,0.95,0.95))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,10.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,50.0)
	   ' We don't want self-cast shadows for the ball mesh 
	   iMeshRenderModeSet(BallMesh,D3DBLEND_SRCALPHA,D3DBLEND_INVSRCALPHA,True,True)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,2.0)
	
	   ' Refrction is an extension of environment map reflection really.
	   ' So, enable envmap functionality first (see envmap examples For details)
	   Envmap=iEnvMapCreate("default_res\\envmap.dds")
	   iMeshEnvMapReceiverAdd(BallMesh,Envmap)
	   iMeshEnvMapBlendSet(BallMesh,0.99)
	   ' The Next line actually enables refraction.
	   iMeshEnvMapGlassSet(BallMesh,0.4)
	
	   iEnvMapUpdateRateSet(Envmap,50.0)
	
	   iSkyBoxEnvMapCasterAdd(SkyBox)
	   iMeshEnvMapCasterAdd(TerrainMesh)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	EndMethod

	Method _Run()
	   Local ballLocation:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,ballLocation)
	   iCameraLookAt(Camera,ballLocation,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraOrientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraOrientation)
	   iVectorRotate(acceleration,acceleration,cameraOrientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   iEnvMapFocusSet(Envmap,ballLocation)
	   iWindowTitleSet(iFrameRate(True))	
	EndMethod
EndType
