' Ball-Type joint demo.
' Arrow keys apply forces To the linked quads.
' [Ctrl] key reverses gravity force direction.
' [A] And [Z] apply some torque about joint's local Y axis.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global QuadA:BODY
Global QuadAMesh:MESH
Global QuadAMeshShadow:MESH
Global QuadB:BODY
Global QuadBMesh:MESH
Global QuadBMeshShadow:MESH
Global QuadATerrain:BODYBODY
Global QuadBTerrain:BODYBODY
Global Joint:JOINT

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   ' Create two identical quad-shaped objects And set thier initial
	   ' location so that their surfaces 'touch' at (0.0,5.0,0.0).
	
	   QuadA=iBodySGCreate("default_res\\quad_.spg",0.13)
	   QuadAMesh=iBodyMeshCreate("default_res\\quad.x",QuadA)
	   QuadAMeshShadow=iMeshBodyShadowCasterCreate("default_res\\quad.x",QuadA,50.0)
	   iBodyLocationSet(QuadA,_D3DXVECTOR3(0.0,6.0,0.0),True)
	   iBodyDampingSet(QuadA,0.0,0.999,0.0,0.995)
	
	   QuadB=iBodySGCreate("default_res\\quad_.spg",0.13)
	   QuadBMesh=iBodyMeshCreate("default_res\\quad.x",QuadB)
	   QuadBMeshShadow=iMeshBodyShadowCasterCreate("default_res\\quad.x",QuadB,50.0)
	   iBodyLocationSet(QuadB,_D3DXVECTOR3(0.0,4.0,0.0),True)
	   iBodyDampingSet(QuadB,0.0,0.999,0.0,0.995)
	
	   ' Create a ball-Type joint at quads' contact point. The two objects will be
	   ' linked To each other at the specified point.
	
	   Joint = iJointBallCreate(QuadA,QuadB,_D3DXVECTOR3(0.0,5.0,0.0))
	
	   ' Set joint rotational limits (stops). See iJointBallXStopsSet() documentation
	   ' in the reference.txt file For important details about setting joint stops!
	
	   iJointBallXStopsSet(Joint,-90.0,90.0,1500000.0,80000.0)
	   iJointBallYStopsSet(Joint,-90.0,90.0,1500000.0,80000.0)
	   iJointBallZStopsSet(Joint,0.0,0.0,1500000.0,80000.0)
	
	   ' Set joint angular motor To Null (no torque, no rotational friction).
	
	   iJointBallXAMotorSet(Joint,0.0,0.0,1.0)
	   iJointBallYAMotorSet(Joint,0.0,0.0,1.0)
	   iJointBallZAMotorSet(Joint,0.0,0.0,1.0)
	
	   ' Enable collision detection  response For the two quads And the terrain
	
	   QuadATerrain=iBodyBodyCreate(QuadA,Terrain)
	   iBodyBodyFrictionSet(QuadATerrain,10.0)
	   QuadBTerrain=iBodyBodyCreate(QuadB,Terrain)
	   iBodyBodyFrictionSet(QuadBTerrain,10.0)
	
	   ' Enable bodies And show meshes
	
	   iBodyEnable(Terrain)
	   iBodyEnable(QuadA)
	   iBodyEnable(QuadB)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(QuadAMesh)
	   iMeshShow(QuadAMeshShadow)
	   iMeshShow(QuadBMesh)
	   iMeshShow(QuadBMeshShadow)
	EndMethod

	Method _Run()
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(QuadA,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(QuadA,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		 iBodyAccelerationApply(QuadA,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(QuadA,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	   iBodyAccelerationApply(QuadB,_D3DXVECTOR3(0.0,-9.80665,0.0))
	
	   ' Apply some angular motor To Local Y axis on [A], [Z] key presses
	
	   If (iKeyDown(DIK_A)) 
		 iJointBallYAMotorSet(Joint,200.0,5.0,0.2)
	   Else If (iKeyDown(DIK_Z))
		 iJointBallYAMotorSet(Joint,-200.0,5.0,0.2)
	   Else
		 iJointBallYAMotorSet(Joint,0.0,0.0,1.0)
	   EndIf
	EndMethod
EndType
