' Simple client/server multiplayer game. Boot other players out of the board!
' Up To four players can play, any number of connected users can watch the match.
' Watching users can get control of a ball when it falls And its owner loses it.
' Who hosts the game can only watch.
' Use arrow keys To control your ball.

' NOTE: in this commented source we assume you understand the simpler
'       NetBallGame And NetChat code examples. Only specific code is
'       explained here.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Const MAX_PLAYERS:Int = 4

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Board:BODY
Global BoardMesh:MESH
Global Ball:BODY[MAX_PLAYERS]
Global BallMesh:MESH[MAX_PLAYERS]
Global BallMeshShadow:MESH[MAX_PLAYERS]
Global BallBoard:BODYBODY[MAX_PLAYERS]
Global BallBall:BODYBODY[(MAX_PLAYERS*MAX_PLAYERS - MAX_PLAYERS)/2]

Global FontWhite:FONT
Global StatsString:String
Global MessageString:String

Global RunningAsServer:Int
Global RunningAsClient:Int

' network synchronization Object variable arrays
Global BallSync:BODYSYNC[MAX_PLAYERS]
Global LeftKeySync:FLOATSYNC[MAX_PLAYERS]
Global RightKeySync:FLOATSYNC[MAX_PLAYERS]
Global UpKeySync:FLOATSYNC[MAX_PLAYERS]
Global DownKeySync:FLOATSYNC[MAX_PLAYERS]

' shared Float variable arrays
Global LeftKeyState:Float[MAX_PLAYERS]
Global RightKeyState:Float[MAX_PLAYERS]
Global UpKeyState:Float[MAX_PLAYERS]
Global DownKeyState:Float[MAX_PLAYERS]

' current controlled ball
Global MyBall:Int

' list of players that control the balls (unused in client mode)
Global BallPlayer:Int[MAX_PLAYERS]

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Local i:Int
	
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-15.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Board=iBodyCreate("default_res\\board_0.97.ply")
	   BoardMesh=iBodyMeshCreate("default_res\\board.x",Board)
	
	   ' For each possible player, we are going To create one ball from the same
	   ' resource (ball.x file) And change ball textures To custom ones afterwards.
	   ' To do so, we need To temporarily disable automatic texture sharing.
	   iMeshTextureSharingSet(False)
	   For i=0 To MAX_PLAYERS-1
	      ' For each player, create a ball body And attach a visible mesh And a shadow To it
	      Ball[i]=iBodySGCreate("default_res\\ball_.spg",0.13)
	      BallMesh[i]=iBodyMeshCreate("default_res\\ball.x",Ball[i])
	      BallMeshShadow[i]=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball[i],10.0)
	      ' Change current ball texture To an unique one. First ball will be
	      ' Default black, second will be red, third yellow And fourth blue.
	      If (i = 1) iMeshTextureChange(BallMesh[i],"default_res\\ballr.jpg")
	      If (i = 2) iMeshTextureChange(BallMesh[i],"default_res\\bally.jpg")
	      If (i = 3) iMeshTextureChange(BallMesh[i],"default_res\\ballb.jpg")
	      ' Set initial location so that the four balls don't overlap
	      iBodyLocationSet(Ball[i],_D3DXVECTOR3(Float(i)*2.5-3.75,5.0,0.0),True)
	      ' Set some damping, To ensure the balls tend To stop when Not subject To forces.
	      iBodyDampingSet(Ball[i],0.0,0.999,0.0,0.995)
	
	      ' For each ball, activate ball-board collision detection, And set friction.
	      BallBoard[i]=iBodyBodyCreate(Ball[i],Board)
	      iBodyBodyFrictionSet(BallBoard[i],10.0)
	
	      ' The following tricky loop activates collision
	      ' detection For all possible ball couples. 
	      Local k:Int
	      Local m:Int = 0
	      For k=0 To MAX_PLAYERS-1
	         If (k < i)
	            BallBall[m]=iBodyBodyCreate(Ball[i],Ball[k])
	            iBodyBodyFrictionSet(BallBall[m],3.0)
	            m:+1
	         EndIf
	      Next
	   Next
	
	   ' Re-enable automatic texture sharing
	   iMeshTextureSharingSet(True)
	
	   ' Enable board And balls bodies
	   iBodyEnable(Board)
	   For i=0 To MAX_PLAYERS-1
		 iBodyEnable(Ball[i])
	   Next
	
	   ' Reset message strings
	   StatsString=""
	   MessageString=""
	
	   ' Let the user decide whether To run as server Or client
	   RunningAsServer = False
	   RunningAsClient = False
	   If (iMessageBoxYesNo("Do you want to host the game?",0))
	      If (iNetServerStart("Ball Arena Game","The Server",25857,$BDBDBDBD,10000))
			RunningAsServer = True
	      Else
			iMessageBox("Could not start the server!")
		 EndIf
	   Else
	      If (iNetClientStart("","The Client",$BDBDBDBD,10000,"NetBallArena",-1,0,Null))
			RunningAsClient = True
	      Else
			iMessageBox("Could not start the client!")
		 EndIf
	   EndIf
	
	   ' MyBall variable keeps track of the ball currently controlled by
	   ' the Local player (0 = black, 1 = red, etc). Initially, the Local
	   ' client player doesn't own a ball, so set MyBall to -1. The server
	   ' never owns a ball.
	   MyBall = -1
	
	   ' On the server, we keep track of each player owning a ball by
	   ' maintaining an array of player's IDs. BallPlayer[0] is the ID
	   ' of the player that controls the black ball, BallPlayer[1] is
	   ' the ID of the player that controls the red ball And so on.
	   ' The following code simply sets all IDs To zero, as nobody
	   ' owns a ball, initially.
	   If (RunningAsServer)
	      For i=0 To MAX_PLAYERS-1
	         BallPlayer[i] = 0
	      Next
	   EndIf
	
	   For i=0 To MAX_PLAYERS-1
	      ' Each connected computer provides its own keyboard. In order To bind
	      ' each ball To a different remote keyboard, we must provide a separate
	      ' set of (synched) Float variables And floatsync objects For each ball.
	      ' For example, LeftKeyState[0] is the Float variable that represents the
	      ' Left arrow key state associated with the black ball.
	      ' Initially, we reset all Float variables To 0.0f (no keys pressed anywhere).
	      LeftKeyState[i] = 0.0
	      RightKeyState[i] = 0.0
	      UpKeyState[i] = 0.0
	      DownKeyState[i] = 0.0
	      ' For each synched Float variable we must create a floatsync
	      LeftKeySync[i] = iNetFloatSyncCreate(LeftKeyState[i],0.109,2.0,0.5)
	      RightKeySync[i] = iNetFloatSyncCreate(RightKeyState[i],0.109,2.0,0.5)
	      UpKeySync[i] = iNetFloatSyncCreate(UpKeyState[i],0.109,2.0,0.5)
	      DownKeySync[i] = iNetFloatSyncCreate(DownKeyState[i],0.109,2.0,0.5)
	      ' For each ball we must create a bodysync
	      BallSync[i] = iNetBodySyncCreate(Ball[i],1.0)
	
	      ' NOTE: we have set floatsync And bodysync frequency so that they tend
	      ' Not To send all data at the same time. This is To keep the packets small
	      ' And evenly distributed over time, as data that are sent simultaneously
	      ' are grouped And sent as a unique bigger packet.
	   Next
	
	   ' Synchronize Local clock with the server
	   iNetClientReset()
	
	   ' Hide load screen And show the meshes
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(BoardMesh)
	   For i=0 To MAX_PLAYERS-1
	      iMeshShow(BallMesh[i])
	      iMeshShow(BallMeshShadow[i])
	   Next
	EndMethod

	Method _Run()
	   Local i:Int
	
	   If (RunningAsServer)
	      ' This If() section is executed only on the server machine.
	      ' It is never executed on a client, Or If the network system isn't running.
	
	      ' In this simple game, the server can only receive one single message Type
	      ' from a client: a ball request, when a remote user presses [B].
	
	      ' So, the following code checks, at every DLL loop, If a message has arrived.
	      ' If so, the ID of the sender is assigned To the first empty slot in the
	      ' BallPlayer[] array, And a message with the assigned ball number is sent
	      ' back To the sender.
	
	      ' If no empty slot is found, it means that all balls are currently assigned
	      ' To other players, so we simply ignore the request.
	
	      Local messageplayer:Int
	      If (iNetData(Null,Null,Varptr(messageplayer)))
	         ' A message has arrived, 'messageplayer' is the sender's ID
	         For i=0 To MAX_PLAYERS-1
	            If Not BallPlayer[i]
	               ' BallPlayer[i] is unassigned, assign 'messageplayer' to it
	               BallPlayer[i] = messageplayer
	               ' Build a one-Byte message with assigned ball number
	               Local onebyte:Byte = i
	               ' Send the message back To 'messageplayer'
	               iNetDataSend(onebyte,1,$00000001,messageplayer,True)
	               ' Break the For() loop, as we are done
	               Exit
	            EndIf
	         Next
	         ' We have finished processing the message, remove it
	         iNetDataDiscard()
	      EndIf
	   EndIf

	   If (RunningAsClient)
	      ' This If() section is executed only on client machines.
	      ' It is never executed on the server, Or If the network system isn't running.
	
	      ' The following code terminates the Local client application If
	      ' the connection with the server has been lost For some reason.
	      If (iNetSessionTerminated())
	         iMessageBox("Connection lost.")
	         iExit()
	      EndIf
	
	      If (MyBall < 0)
	         ' MyBall is -1. It means that the Local client doesn't own a ball.
	
	         ' Set message String To a message For the Local user
	         MessageString="Press [B] to request a ball"
	
	         ' Check whether the Local [B] key is pressed
	         If (iKeyClick(DIK_B))
	            ' The [B] key has been pressed, send ball request message To the server.
	            ' It is a simple zero byte. The server will understand that it is a ball
	            ' request because clients send only ball requests in this simple game.
	            Local onebyte:Byte = 0
	            iNetDataSend(onebyte,1,$00000001,$00000000,True)
	         EndIf
	
	         ' Check If a message from the server has arrived. If so it is a
	         ' ball-request-accepted message.
	         ' We can only receive that kind of message from the server,
	         ' While we do Not own a ball (Local player's ID not listed in BallPlayer[]
	         ' array, on the server, And MyBall = -1 on Local machine).
	         Local yourballis:String
	         Local buf:Byte Ptr
	         If (iNetData(Varptr(buf),Null,Null))
			  yourballis=String.FromCString(buf)
	            ' A message has arrived.
	            ' The 'yourballis' points to the received message now.
	            ' In this game it is a simple Byte: the number of the ball the server
	            ' assigned To us. So copy the ball number To MyBall variable. 
	            MyBall = yourballis.ToInt()
	            ' We have finished processing the message, remove it
	            iNetDataDiscard()
	         EndIf
	      Else
	         ' MyBall is greater Or equal To zero. It means that the Local client
	         ' owns a ball (0 For black, 1 For red, And so on).
	
	         ' Set message String To a message For the Local user
	         If (MyBall = 0) MessageString="Your ball is BLACK"
	         If (MyBall = 1) MessageString="Your ball is RED"
	         If (MyBall = 2) MessageString="Your ball is YELLOW"
	         If (MyBall = 3) MessageString="Your ball is BLUE"
	
	         ' Check If a message from the server has arrived. If so it is a
	         ' you-lost-your-ball message.
	         ' We can only receive that kind of message from the server,
	         ' While we own a ball (Local player's ID listed in BallPlayer[]
	         ' array, on the server, And MyBall >= 0 on Local machine).
	         If (iNetData(Null,Null,Null))
	            ' A message has arrived, we have been booted out.
	            ' Set MyBall accordingly
	            MyBall = -1
	            ' We have finished processing the message, remove it
	            iNetDataDiscard()
	         EndIf
	      EndIf

	      ' Allow Local client user To reset network synchronization by pressing space
	      If (iKeyClick(DIK_SPACE)) iNetClientReset()
	
	      ' If the Local client owns a ball, Then update the ball-specific set of synched
	      ' variables. Nobody Else in the network owns this ball currently, so only the
	      ' Local player will alter this ball-specific variable set (...KeySync[MyBall]).
	      ' That is, only the Local player (his/her Local keyboard) can control the
	      ' MyBall ball, currently. Remember, synched variables will change on all
	      ' connected computers as soon as iNetFloatSyncUpdate() is called. 
	      If (MyBall >= 0)
	         iNetFloatSyncUpdate(LeftKeySync[MyBall],Float(iKeyDown(DIK_LEFT)))
	         iNetFloatSyncUpdate(RightKeySync[MyBall],Float(iKeyDown(DIK_RIGHT)))
	         iNetFloatSyncUpdate(UpKeySync[MyBall],Float(iKeyDown(DIK_UP)))
	         iNetFloatSyncUpdate(DownKeySync[MyBall],Float(iKeyDown(DIK_DOWN)))
	      EndIf
	
	      ' Visualize Local client stats And messages
	      iPrintCentered("Press SPACE to re-sync and re-compute minimum lag",_D3DXVECTOR2(0.0,11.4),_D3DXVECTOR2(0.7,0.5),0.1,FontWhite,0.0)
		 StatsString="LAG: "+iNetMinimumLag()+" - SYNC: "+iNetClientSync()
	      iPrintCentered(StatsString,_D3DXVECTOR2(0.0,10.7),_D3DXVECTOR2(0.7,0.5),0.1,FontWhite,0.0)
	      iPrintCentered(MessageString,_D3DXVECTOR2(0.0,-11.0),_D3DXVECTOR2(1.0,0.5),0.1,FontWhite,0.0)
	   EndIf

	   ' The following code orients the camera so that it tracks the currently owned ball.
	   ' If no ball is owned, orient the camera toward the board center To simply watch.
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   If (MyBall >= 0)
	      iBodyLocationCM(Ball[MyBall],cameratarget)
	   Else
	      cameratarget = _D3DXVECTOR3(0.0,0.0,0.0)
	   EndIf
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   For i=0 To MAX_PLAYERS-1
	      ' For each ball, compute forces determined by the state of its
	      ' specific set of synched Float variables (network key presses). 
	      Local acceleration:D3DXVECTOR3
	      acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	      If (LeftKeyState[i]) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	      If (RightKeyState[i]) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	      If (UpKeyState[i]) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	      If (DownKeyState[i]) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	      acceleration.y = 0.0
	      ' Apply resulting force
	      iBodyAccelerationApply(Ball[i],acceleration)
	
	      ' Apply gravity, always
	      iBodyAccelerationApply(Ball[i],_D3DXVECTOR3(0.0,-9.80665,0.0))
	
	      ' Check whether the current ball is in game
	      Local p:D3DXVECTOR3=_D3DXVECTOR3()
	      iBodyLocation(Ball[i],p)
	      If (p.y < -100.0)
	         ' The current ball is below a certain Y coordinate.
	         ' It means that it has been booted out of the board And has fallen.
	
	         ' Reset ball physics (zero velocity, spin) And put it back over the board.
	         iBodyPhysicsReset(Ball[i])
	         iBodyLocationSet(Ball[i],_D3DXVECTOR3(Float(i)*2.5-5.0,5.0,0.0),True)
	
	         ' If running on the server, check If the ball was owned by somebody
	         If (RunningAsServer)
	            If (BallPlayer[i])
	               ' BallPlayer[i] is non-zero, the ball was controlled.
	               ' BallPlayer[i] is the ID of the owner.
	               ' So send a you-lost-your-ball message To that remote player.
	               Local onebyte:Byte = 0
	               iNetDataSend(onebyte,1,$00000001,BallPlayer[i],True)
	               ' Set player's ID in the BallPlayer[] array to zero,
	               ' so it gets available again
	               BallPlayer[i] = 0
	            EndIf
	         EndIf
	      EndIf
	   Next
	EndMethod
	
	Method _Exit()
	   ' The network system must always be stopped before exiting the application
	   iNetStop()
	EndMethod
EndType
