' Network playable version of the keyboard controlled ball demo.
' Arrow keys apply forces To the ball.
' [Ctrl] key reverses gravity force direction.
' Each client can control the same ball (weird!). The server can only watch.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY

Global FontWhite:FONT
Global MessageString:String

' flags To keep track of current running mode
Global RunningAsServer:Int
Global RunningAsClient:Int

' network synchronization Object variables
Global BallSync:BODYSYNC
Global CtrlKeySync:FLOATSYNC
Global LeftKeySync:FLOATSYNC
Global RightKeySync:FLOATSYNC
Global UpKeySync:FLOATSYNC
Global DownKeySync:FLOATSYNC

' shared Float variables
Global CtrlKeyState:Float 
Global LeftKeyState:Float 
Global RightKeyState:Float 
Global UpKeyState:Float
Global DownKeyState:Float

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,15.0,30.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,50.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   ' A typical network application is built as two separate versions: one For the
	   ' server And one For the clients. In our examples, we implement a single application
	   ' that can be run either as a server Or a client software.
	
	   ' The following code allows the user To decide whether To run the software as
	   ' server that hosts the game.
	
	   ' If so, the RunningAsServer flag will be True And client-specific code will
	   ' never be executed (note: RunningAsServer isn't used in this example).
	
	   ' If Not so, the RunningAsClient flag will be True And server-specific code will
	   ' never be executed. Only common And client-specific code will.
	
	   ' RunningAsServer And RunningAsClient flags can never be both TRUE.
	
	   RunningAsServer = False
	   RunningAsClient = False
	   If (iMessageBoxYesNo("Do you want to host the game?",0))
	      ' Try starting the system in server mode
	      If (iNetServerStart("Ball Game","The Server",25857,$BBBBBBBB,10000))
			RunningAsServer = True
	      Else
			iMessageBox("Could not start the server!")
		 EndIf
	   Else
	      ' Try starting the system in client mode
	      If (iNetClientStart("","The Client",$BBBBBBBB,10000,"NetBallGame",-1,0,Null))
			RunningAsClient = True
	      Else
			iMessageBox("Could not start the client!")
		 EndIf
	   EndIf
	
	   ' Initialize Float variables And the message String
	   CtrlKeyState = 0.0
	   LeftKeyState = 0.0
	   RightKeyState = 0.0
	   UpKeyState = 0.0
	   DownKeyState = 0.0
	   MessageString=""
	
	   ' The following 6 lines of code specify what bodies And Float variables have To be
	   ' synchronized. Note that the initialization below will be identically executed on
	   ' all connected computers, including the server. All connected computers must know
	   ' what objects are synchronized And the sequential order of i...SyncCreate() calls.
	
	   ' To tell the engine that a body has To be synchronized, you must create, on all
	   ' machines, a so called 'bodysync' object. The bodysync object will then automatically
	   ' take care of sending synch data from the server And receiving it on the clients.
	   ' The Next line specifies 'Ball' as the body to synchronize and 0.5f (half a second)
	   ' as synchronization frequency (body state will be adjusted twice a second). It
	   ' Then stores the created bodysync Object in a BODYSYNC* variable named 'BallSync'.
	   BallSync = iNetBodySyncCreate(Ball,0.5)
	
	   ' You tell the engine that a Float variable has To be synchronized by creating, on all
	   ' machines, a so called 'floatsync' object. The foatsync object will then automatically
	   ' take care of sending synch data from the machine that executes iNetFloatSyncUpdate()
	   ' To the server, which in turn will send it To all connected clients.
	   ' As a result, the Float variable associated with the floatsync Object will be
	   ' set the same on all connected machines, at regular intervals.
	   ' The Next line specifies 'CtrlKeyState' as the float variable to synchronize and
	   ' 0.15f seconds as synchronization frequency (variable state will be shared 6.6
	   ' times per second). It Then stores the created floatsync Object in a FLOATSYNC*
	   ' variable named 'CtrlKeySync'.
	   CtrlKeySync = iNetFloatSyncCreate(CtrlKeyState,0.15,2.0,0.5)
	   ' The four lines below do the same For the Float variables we use To
	   ' store And share the state of Left, Right, Up, Down arrow keys.
	   LeftKeySync = iNetFloatSyncCreate(LeftKeyState,0.15,2.0,0.5)
	   RightKeySync = iNetFloatSyncCreate(RightKeyState,0.15,2.0,0.5)
	   UpKeySync = iNetFloatSyncCreate(UpKeyState,0.15,2.0,0.5)
	   DownKeySync = iNetFloatSyncCreate(DownKeyState,0.15,2.0,0.5)
	
	   ' When a client joins a session, that is, after iNetClientStart() call succeeds,
	   ' the first thing To do is making sure that the Local machine's clock matches
	   ' the server machine's clock. This is necessary to allow bodysync and floatsync
	   ' objects To work properly behind the scenes.
	   ' The Next line synchronizes the Local clock with the server And properly resets
	   ' all bodysync And floatsync objects.
	   iNetClientReset()
	   ' Note that calling iNetClientReset() on the server Or when no network system is
	   ' running, has no effect, so we can just call it here without checking whether
	   ' RunningAsClient is True Or not. Strict coding would check that anyway... 
	
	   ' Hide load screen And show the meshes
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	EndMethod
	
	Method _Run()
	   If (RunningAsClient)
	      ' This If() section is executed only on client machines.
	      ' It is never executed on the server, Or If the network system isn't running.
	
	      ' You cannot modify a synchronized Float variable by simply assigning
	      ' a value To it. Instead, you must call, at every DLL loop, iNetFloatSyncUpdate()
	      ' For the floatsync Object associated with it.
	      ' The following 5 lines set -1.0f To each Float variable when a certain
	      ' key is pressed, And 0.0f when it is not. This will update the variables
	      ' on all connected machines automatically, behind the scenes.
	      iNetFloatSyncUpdate(CtrlKeySync,Float(iKeyDown(DIK_LCONTROL)))
	      iNetFloatSyncUpdate(LeftKeySync,Float(iKeyDown(DIK_LEFT)))
	      iNetFloatSyncUpdate(RightKeySync,Float(iKeyDown(DIK_RIGHT)))
	      iNetFloatSyncUpdate(UpKeySync,Float(iKeyDown(DIK_UP)))
	      iNetFloatSyncUpdate(DownKeySync,Float(iKeyDown(DIK_DOWN)))
	      ' NOTE: the '(float)', above, means 'convert whatever follows to a float'.
	      ' For example: take 'iKeyDown(DIK_LCONTROL)' which is a BOOL and convert
	      ' it To a float. That is, If 'iKeyDown(DIK_LCONTROL)' returns FALSE, then
	      ' convert it To 0.0f. If it returns True Then convert it To -1.0f.
	
	      ' The Next line allows the user To manually reset the synchronization by
	      ' pressing space. This is useful To Try To regain an optimal client/server
	      ' synchronization after a poor connection caused packet losses And latency.
	      ' A real world implementation should automatically call iNetClientReset()
	      ' whenever iNetClientSync() returns values below -0.5f Or above 0.5f
	      If (iKeyClick(DIK_SPACE)) iNetClientReset()
	
	      ' Print network synchronization statistics
	      MessageString="LAG: "+iNetMinimumLag()+" - SYNC: "+iNetClientSync()
	      iPrintCentered("Press SPACE to re-sync and re-compute minimum lag",_D3DXVECTOR2(0.0,11.4),_D3DXVECTOR2(0.7,0.5),0.1,FontWhite,0.0)
	      iPrintCentered(MessageString,_D3DXVECTOR2(0.0,10.7),_D3DXVECTOR2(0.7,0.5),0.1,FontWhite,0.0)
	   EndIf
	
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   ' In a non-network simulation we would normally generate forces as a direct
	   ' consequence of key presses (If (iKeyDown(DIK_LEFT))...).
	   ' In this network simulation we must rely on synchronized Float variables
	   ' instead. Remember that this code may run on a machine where no keys
	   ' are pressed (like the server, Or an idle client). Only synched floats will
	   ' tell us whether somebody has pressed a key on some remote client machine.
	   If (LeftKeyState) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (RightKeyState) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (UpKeyState) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (DownKeyState) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	   If (CtrlKeyState)
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   EndIf
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)

	   iWindowTitleSet(iFrameRate(True))	
	EndMethod

	Method _Exit()
	   ' The network system must always be stopped before exiting the application
	   iNetStop()
	EndMethod
EndType