' Server To client file upload demo
' The server waits Until a client joins the session. As soon as a client
' is connected, the server uploads a file To the remote client machine.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global FontWhite:FONT

Global BallMesh:MESH

Global MessageString:String

Global RunningAsServer:Int
Global RunningAsClient:Int
Global WaitPhase:Int
Global TransferPhase:Int
Global CompletePhase:Int

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   ' We are going To render a spinning mesh in the background To show
	   ' that file transfers can happen without halting the simulation
	   BallMesh=iMeshCreate("default_res\\ball.x")
	   iMeshLocationSet(BallMesh,_D3DXVECTOR3(0.0,0.0,3.5))
	
	   ' Initialize flags
	   RunningAsServer = False
	   RunningAsClient = False
	   WaitPhase = True ' start both server And client in wait mode
	   TransferPhase = False
	   CompletePhase = False
	
	   ' Let the user decide whether To launch the demo in server Or client mode
	   If (iMessageBoxYesNo("Run the demo in server mode?",0))
	      ' Start the network system in server mode.
	      If (iNetServerStart("File Upload Demo","The Server",25857,$BABABABA,30000))
		    RunningAsServer = True
	      Else
	         ' an error occurred, terminate
	         iMessageBox("Could not start the server!")
	         iExit()
	      EndIf
	   Else
	      ' Start the network system in client mode.
	      If (iNetClientStart("","The Client",$BABABABA,30000,"NetFileUpload",-1,0,Null))
		    RunningAsClient = True
	      Else
	         ' an error occurred Or the user canceled the connection dialog, terminate
	         iMessageBox("Could not start the client!")
	         iExit()
	      EndIf
	   EndIf
	
	   ' Show the background mesh
	   iMeshShow(BallMesh)
	EndMethod
	
	Method _Run()
	   If (RunningAsServer)
	      ' This If() section is executed only on the server machine.
	      ' It is never executed on a client, Or If the network system isn't running.
	
	      If (WaitPhase)
	         ' This If() section is executed only While the server is waiting
	         ' For a client To join the session And connect To the network.
	
	         ' Tell Local user about it
	         MessageString="Waiting for a client..."
	
	         ' The following code, every two seconds, sends To any connected client an
	         ' "are-you-ready" message. Nothing happens If no clients are connected yet.
	         ' The message is a dummy, Null byte. The message Type is 0x00000002.
	         ' See RunningAsClient section To see what happens when the client receives
	         ' this message.
	         Global timer:Float = 0.0
	         If (timer > 2.0)
	            Local onebyte:Byte = 0
	            iNetDataSend(onebyte,1,$00000002,$00000000,True)
	            timer = 0.0
	         Else
			  timer :+ iFrq()
		    EndIf
	
	         ' The following code, at every DLL loop, polls the message
	         ' buffer To see If a message from a client has arrived
	         Local messagetype:Int
	         If (iNetData(Null,Varptr(messagetype),Null))
	            ' iNetData() returned non-zero. It means that a message has arrived
	            ' The 'messagetype' variable is the type of message.
	            If (messagetype = $00000003)
	               ' Message Type is 0x00000003. It is the "I'm-ready" message.
	               ' It means that a client joined the network, received our
	               ' 'are-you-ready' message and confirmed it.
	               ' So, switch To file transfer phase.
	               WaitPhase = False
	               TransferPhase = True
	               MessageString="Uploading..."
	            EndIf
	            ' Don't forget to remove the message from the buffer once you are done with it
	            iNetDataDiscard()
	         EndIf
	      EndIf
	      If (TransferPhase)
	         ' This If() section is executed only While the server
	         ' is uploading the file To the a client
	
	         Local filepath:String
	         Local filebuffer:String
	
	         ' Fill the 'filepath' string with the full path to
	         ' the mirrorfloor.jpg image file, the file we want To upload.
	         ' We are sending a 26k file from the default_res folder.
	         ' Don't forget to increase the network buffer size if you
	         ' are going To send a bigger file! See iNetServerStart() And
	         ' iNetClientStart() Function calls in the Init() callback above.
	         iGlobalPathMake(filepath,"default_res\\mirrorfloor.jpg")
	
	         ' Open the file To upload so we can read it
		    Local hFile:TStream=ReadFile(filepath)

		    If (hFile)
	            ' Read And store the file in the allocated buffer
			  filebuffer=ReadString(hFile,StreamSize(hFile))
	            ' Close the file
			  CloseStream(hFile)
	            ' Send the buffer (the whole file) as an unique, big, guaranteed message.
	            ' Sending big guaranteed message isn't recommended, but we are doing it anyway to
	            ' keep the code simple. The best would be sending the file 4k Or so at a time.
	            ' It would also allow you, For example, To stretch a sprite proportionally
	            ' To the number of packets sent, To implement an upload progress bar.
	            iNetDataSend(filebuffer,filebuffer.length,$00000001,$00000000,True)
	            ' Now that the message has been sent, the buffer is no longer
	            ' necessary. Release the memory block.
	            filebuffer=Null
	            ' We are done, switch To upload-complete phase
	            TransferPhase = False
	            CompletePhase = True
	            MessageString="Upload complete. Press [Esc] to exit."
	         Else
	            ' If we are here, the file To upload could Not be opened.
	            iMessageBox("Could not find the file to upload!")
	            iExit()
	         EndIf
	      EndIf
	      If (CompletePhase)
	         ' This If() section is executed only after the server
	         ' has completed the download. Nothing To do here...
	      EndIf
	   EndIf
	
	   If (RunningAsClient)
	      ' This If() section is executed only on the client machine.
	      ' It is never executed on the server, Or If the network system isn't running.
	
	      If (WaitPhase)
	         ' This If() section is executed only While the client is waiting
	         ' For the 'are-you-ready' message from the server
	
	         ' Tell Local user about it
	         MessageString="Waiting for the server..."
	
	         ' The following code, at every DLL loop, polls the message
	         ' buffer To see If a message from the server has arrived.
	         Local messagetype:Int
	         If (iNetData(Null,Varptr(messagetype),Null))
	            ' iNetData() returned non-zero. It means that a message has arrived
	            ' The 'messagetype' variable is the type of message.
	            If (messagetype = $00000002)
	               ' Message Type is 0x00000002. It is the "are-you-ready" message.
	               ' So, send the "I'm-ready" message To the server, as confirmation.
	               Local onebyte:Byte = 0
	               iNetDataSend(onebyte,1,$00000003,$00000000,True)
	               ' Done. Switch To file transfer (file-receive) phase.
	               WaitPhase = False
	               TransferPhase = True
	               MessageString="Downloading..."
	            EndIf
	            ' Don't forget to remove the message from the buffer once you are done with it
	            iNetDataDiscard()
	         EndIf
	      EndIf
	      If (TransferPhase)
	         ' This If() section is executed only While the client
	         ' is receiving the file from the a server
	
	         ' The following code, at every DLL loop, polls the message
	         ' buffer To see If a message from the server has arrived.
	         Local messagetype:Int
	         If (iNetData(Null,Varptr(messagetype),Null))
	            ' iNetData() returned non-zero. It means that a message has arrived
	            ' The 'messagetype' variable is the type of message.
	            If (messagetype = $00000001)
	               ' Message Type is 0x00000001. It is the file.
	               ' So, retrieve message length And memory address of the first
	               ' Byte of the message, in the received-message buffer
	               Local filelength:Int
	               Local filebuffer:Byte Ptr
	               filelength = iNetData(Varptr(filebuffer),Null,Null)
	               ' The 'filebuffer' variable points to start of message now,
	               ' that is, To the first Byte of received file.
	               ' The 'filelength' variable is the length of received file.
	
	               Local filepath:String
	
	               ' Fill the 'filepath' string with the full path of the
	               ' file we want To save (the name of downloaded file).
	               ' We are downloading the file To the main folder, the
	               ' folder where the compiled .dll is.
	               iGlobalPathMake(filepath,"downloaded_mirrorfloor.jpg")
	
	               ' Create a New file on the hard-disk with the specified name
				CreateFile(filepath)
				Local hFile:TStream=WriteFile(filepath)
	               If (hFile)
	                  ' Write file data from the memory buffer To the file
				   WriteString(hFile,String.FromBytes(filebuffer,filelength))
	                  ' Close the file
	                  CloseFile(hFile) 
	                  ' We are done, switch To complete-phase
	                  TransferPhase = False
	                  CompletePhase = True
	                  MessageString="Download complete! Press [Esc] to exit."
	               EndIf
	            EndIf
	            ' Don't forget to remove the message from the buffer once you are done with it
	            iNetDataDiscard()
	         EndIf
	      EndIf
	      If (CompletePhase)
	         ' This If() section is executed only after the client
	         ' has succesfully received the file. Nothing To do here...
	      EndIf
	   EndIf
	
	   ' Render the message String
	   iPrintCentered(MessageString,_D3DXVECTOR2(0.0,0.0),_D3DXVECTOR2(1.0,1.0),0.1,FontWhite,0.0)
	
	   ' Spins the background mesh
	   iMeshOrientationAdd(BallMesh,_D3DXVECTOR3(0.0,1.0,0.0),30.0)
	EndMethod
	
	Method _Exit()
	   ' The network system must always be stopped before exiting the application
	   iNetStop()
	EndMethod
EndType
