'A modified version of the BallGame project that
'demonstrates underwater camera's post-processing effect


Strict
Framework _3Impact.Lib

Global CameraA:CAMERA
Global CameraB:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY

Global CameraEffect:CAMERAEFFECT
Global EffectTimeParameter:Float

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()

		DebugLog(iPixelShaderVersion(False) + " | " + iVertexShaderVersion(False))

		' Create And set the first camera. This will be the Upper rendering window, with no camera effect.
		CameraA=iCameraCreate(0.0,0.0,1.0,0.5)
		iCameraLocationSet(CameraA,_D3DXVECTOR3(0.0,3.0,-10.0))
	
		' Create And set the second camera. This will be the Lower rendering window,
		' the same 3d scene with the underwater camera effect applied.
		CameraB = iCameraCreate(0.0,0.5,1.0,0.5)
		iCameraLocationSet(CameraB,_D3DXVECTOR3(0.0,3.0,-10.0))
		
		' Create the effect from the pixel-shader file (open the .fx file with an ASCII
		' editor For details on how this effect works)
		CameraEffect = iCameraEffectCreate("fxShaders\\pp_underwater.fx")
		' Assign the effect To the second camera
		iCameraEffectSet(CameraB,CameraEffect)
		' Initialize the parameter tha we'll sent to the pixel-shader (see Run() loop, below)
		EffectTimeParameter = 1.0
		
		iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
		
		LoadingMessage = iSpriteCreate("default_res\\sprites\\loading.x",Null)
		iSpriteRender(LoadingMessage,False)
		
		SkyBox = iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
		
		Terrain = iBodyCreate("default_res\\terrain_5.00.ply")
		TerrainMesh = iBodyMeshCreate("default_res\\terrain.x",Terrain)
		
		Ball = iBodySGCreate("default_res\\ball_.spg",0.13)
		BallMesh = iBodyMeshCreate("default_res\\ball.x",Ball)
		iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
		iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
		
		BallMeshShadow = iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
		
		BallTerrain = iBodyBodyCreate(Ball,Terrain)
		iBodyBodyFrictionSet(BallTerrain,10.0)
		
		iBodyEnable(Terrain)
		iBodyEnable(Ball)
		
		iSpriteHide(LoadingMessage)
		iSkyBoxShow(SkyBox)
		iMeshShow(TerrainMesh)
		iMeshShow(BallMesh)
		iMeshShow(BallMeshShadow)
	EndMethod

	Method _Run()

		Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
		iBodyLocationCM(Ball,cameratarget)
		iCameraLookAt(CameraA,cameratarget,0.1)	
		iCameraLookAt(CameraB,cameratarget,0.1)	
		
		Local acceleration:D3DXVECTOR3=_D3DXVECTOR3()
		Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
		If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
		If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
		If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
		If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))	

		iCameraOrientation(CameraA,cameraorientation)
		iVectorRotate(acceleration,acceleration,cameraorientation)

		acceleration.y = 0.0

		iBodyAccelerationApply(Ball,acceleration)

		If Not iKeyDown(DIK_LCONTROL)
		   iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
		Else 
		   iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
		EndIf
	
		' Send the 'time' parameter to the camera effect shader
		iCameraEffectValueSet(CameraB,"time",Byte Ptr(Varptr(EffectTimeParameter)),4)
		
		' Update the camera effect parameter
		EffectTimeParameter :+ iFrq()
		
		If iKeyClick(DIK_H)
			iCameraEffectSet(CameraB, Null)
		EndIf
		
		If iKeyClick(DIK_J)
			iCameraEffectSet(CameraB, CameraEffect)
		EndIf
		
	EndMethod
	
	Method _Exit()
	   ' Make sure you always destroy created effects before exiting the application...
	   iCameraEffectDestroy(CameraEffect)
	EndMethod
	
EndType