'A modified version of the BallGame project that
'demonstrates bloom post-processing effect

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY

Global CameraEffect1:CAMERAEFFECT
Global CameraEffect2:CAMERAEFFECT
Global CameraEffect3:CAMERAEFFECT
Global CameraEffect4:CAMERAEFFECT

Global EffectTimeParameter:Float

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()

'		DebugLog(iPixelShaderVersion(False) + " | " + iVertexShaderVersion(False))

		' Create And set the camera
		Camera = iCameraCreate(0.0, 0.0, 1.0, 1.0)
		iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
		' Create all the effects we need (open the .fx file with an ASCII editor For details on how effect effect works)
		CameraEffect1 = iCameraEffectCreate("fxShaders\\pp_brightonly.fx")
		CameraEffect2 = iCameraEffectCreate("fxShaders\\pp_bloomh.fx")
		CameraEffect3 = iCameraEffectCreate("fxShaders\\pp_bloomv.fx")
		CameraEffect4 = iCameraEffectCreate("fxShaders\\pp_combine.fx")
		
		' Assign the effects To the camera, in the order we want them To be processed
		iCameraEffectStack(Camera, Null, 1.0) ' clear stack
		iCameraEffectStack(Camera, CameraEffect1, 0.125) ' pp_brightonly.fx
		iCameraEffectStack(Camera, CameraEffect2, -0.125) ' pp_bloomh.fx
		iCameraEffectStack(Camera, CameraEffect3, -0.125) ' pp_bloomv.fx
		iCameraEffectStack(Camera, CameraEffect2, -0.125) ' pp_bloomh.fx
		iCameraEffectStack(Camera, CameraEffect3, 8.0) ' pp_bloomv.fx
		iCameraEffectStack(Camera, CameraEffect4, 1.0) ' pp_combine.fx
		
		iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
		
		LoadingMessage = iSpriteCreate("default_res\\sprites\\loading.x",Null)
		iSpriteRender(LoadingMessage,False)
		
		SkyBox = iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
		
		Terrain = iBodyCreate("default_res\\terrain_5.00.ply")
		TerrainMesh = iBodyMeshCreate("default_res\\terrain.x",Terrain)
		
		Ball = iBodySGCreate("default_res\\ball_.spg",0.13)
		BallMesh = iBodyMeshCreate("default_res\\ball.x",Ball)
		iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
		iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
		
		BallMeshShadow = iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
		
		BallTerrain = iBodyBodyCreate(Ball,Terrain)
		iBodyBodyFrictionSet(BallTerrain,10.0)
		
		iBodyEnable(Terrain)
		iBodyEnable(Ball)
		
		iSpriteHide(LoadingMessage)
		iSkyBoxShow(SkyBox)
		iMeshShow(TerrainMesh)
		iMeshShow(BallMesh)
		iMeshShow(BallMeshShadow)
	EndMethod

	Method _Run()

		Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
		iBodyLocationCM(Ball,cameratarget)
		iCameraLookAt(Camera, cameratarget, 0.1)
		
		Local acceleration:D3DXVECTOR3=_D3DXVECTOR3()
		Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
		If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
		If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
		If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
		If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))	

		iCameraOrientation(Camera, cameraorientation)
		iVectorRotate(acceleration,acceleration,cameraorientation)

		acceleration.y = 0.0

		iBodyAccelerationApply(Ball,acceleration)

		If Not iKeyDown(DIK_LCONTROL)
		   iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
		Else 
		   iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
		EndIf
	
		' Send texel/pixel conversion tables (used by bloom .fx shaders)
		Local texelKernel:Float[2*13]
		Local pixToTexX:Float
		Local pixToTexY:Float
		
		pixToTexX = 1.0/iDisplayWidth()
		pixToTexY = 0.0
		
		texelKernel[0]=(-6*pixToTexX)
		texelKernel[1]=(-6*pixToTexY)
		texelKernel[2]=(-5*pixToTexX)
		texelKernel[3]=(-5*pixToTexY)
		texelKernel[4]=(-4*pixToTexX)
		texelKernel[5]=(-4*pixToTexY)
		texelKernel[6]=(-3*pixToTexX)
		texelKernel[7]=(-3*pixToTexY)
		texelKernel[8]=(-2*pixToTexX)
		texelKernel[9]=(-2*pixToTexY)
		texelKernel[10]=(-1*pixToTexX)
		texelKernel[11]=(-1*pixToTexY)
		texelKernel[12]=(0*pixToTexX)
		texelKernel[13]=(0*pixToTexY)
		texelKernel[14]=(1*pixToTexX)
		texelKernel[15]=(1*pixToTexY)
		texelKernel[16]=(2*pixToTexX)
		texelKernel[17]=(2*pixToTexY)
		texelKernel[18]=(3*pixToTexX)
		texelKernel[19]=(3*pixToTexY)
		texelKernel[20]=(4*pixToTexX)
		texelKernel[21]=(4*pixToTexY)
		texelKernel[22]=(5*pixToTexX)
		texelKernel[23]=(5*pixToTexY)
		texelKernel[24]=(6*pixToTexX)
		texelKernel[25]=(6*pixToTexY)
		
		iCameraEffectValueSet(Camera, "TexelKernelH", Varptr(texelKernel[0]), (4+4)*13)
		
		pixToTexX = 0.0
		pixToTexY = 1.0/iDisplayHeight()
		
		texelKernel[0]=(-6*pixToTexX)
		texelKernel[1]=(-6*pixToTexY)
		texelKernel[2]=(-5*pixToTexX)
		texelKernel[3]=(-5*pixToTexY)
		texelKernel[4]=(-4*pixToTexX)
		texelKernel[5]=(-4*pixToTexY)
		texelKernel[6]=(-3*pixToTexX)
		texelKernel[7]=(-3*pixToTexY)
		texelKernel[8]=(-2*pixToTexX)
		texelKernel[9]=(-2*pixToTexY)
		texelKernel[10]=(-1*pixToTexX)
		texelKernel[11]=(-1*pixToTexY)
		texelKernel[12]=(0*pixToTexX)
		texelKernel[13]=(0*pixToTexY)
		texelKernel[14]=(1*pixToTexX)
		texelKernel[15]=(1*pixToTexY)
		texelKernel[16]=(2*pixToTexX)
		texelKernel[17]=(2*pixToTexY)
		texelKernel[18]=(3*pixToTexX)
		texelKernel[19]=(3*pixToTexY)
		texelKernel[20]=(4*pixToTexX)
		texelKernel[21]=(4*pixToTexY)
		texelKernel[22]=(5*pixToTexX)
		texelKernel[23]=(5*pixToTexY)
		texelKernel[24]=(6*pixToTexX)
		texelKernel[25]=(6*pixToTexY)
		
		iCameraEffectValueSet(Camera, "TexelKernelV", Varptr(texelKernel[0]), (4+4)*13)
		
	EndMethod
	
	Method _Exit()
	   ' Make sure you always destroy created effects before exiting the application...
		iCameraEffectDestroy(CameraEffect4)
		iCameraEffectDestroy(CameraEffect3)
		iCameraEffectDestroy(CameraEffect2)
		iCameraEffectDestroy(CameraEffect1)
	EndMethod
	
EndType