' Trails demo

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Include "..\common\3impact_cameraman.bmx"

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY

Global CMKD:CameramanLookAtPointKeepDistance_t=New CameramanLookAtPointKeepDistance_t

Global Trail:TRAIL
Global TrailEmitter:TRAILEMITTER

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,17.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,16.0,50.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   Local collisionbodies:BODY[10]
	   collisionbodies[0] = Terrain
	   collisionbodies[1] = Null
	   CMKD.Create(Camera,1.0,15.0,2.0,1.0,0.7,0.35,collisionbodies)
	
	   ' Create the trail Object from a .dds file.
	   ' Set the maximum number of visible trail segments To 3000
	 
	   Trail=iTrailCreate("default_res\\trail.dds",3000)
	   ' Set trail render mode To burn-out
	   iTrailRenderModeSet(Trail,1)
	   ' Set trail color
	   Local col:D3DXVECTOR3 = _D3DXVECTOR3(1.0,0.0,0.0)
	   iTrailColorSet(Trail,col,col)
	   ' Set lifetime For each trail segment. Each trail segment will vanish after 30 seconds.
	   iTrailLifeTimeSet(Trail,30.0)
	   ' Create an emitter For the trial.
	   ' Note that you can create as many emitters as you wish For each trail.
	   TrailEmitter=iTrailEmitterCreate(Trail)
	   ' Set trail emitter size
	   iTrailEmitterScaleSet(TrailEmitter,0.5)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	   ' Don't forget to show the trail object
	   iTrailShow(Trail)
	EndMethod
	Method _Run()
	   Local actorlocation:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,actorlocation)
	
	   CMKD.Update(actorlocation)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   ' Trails are made of connected quadrangles. At each emission, a New
	   ' quadrangle is appended To the previous quadrangle.
	   ' When the emitter orientation is Null, the ending side of the
	   ' newly emitted quadrangle is parallel To the world X axis.
	
	   ' We want To orient our trail emitter so that shared quadrangle edges
	   ' are always perpendicular To ball moving direction.
	
	   ' Define a couple variables we need To properly orient the trail emitter
	   Local velocity:D3DXVECTOR3=_D3DXVECTOR3()
	   Local orientation:D3DXQUATERNION=_D3DXQUATERNION()
	   ' Place the trail emitter at the center of the ball (current ball location)
	   iTrailEmitterLocationSet(TrailEmitter,actorlocation)
	   ' Retrieve the current ball velocity vector.
	   ' Velocity vector points 'forward'. It is basically the ball moving direction vector.
	   ' We'll use it as a reference to orient the emitter.
	   iBodyVelocity(Ball,velocity)
	   ' Compute a quaternion that represents the rotation required To orient
	   ' an imaginary +Z vector like (0.0,0.0,1.0) towards ball moving direction.
	   ' It is basically the orientation we want For the emitter, so that the edge of
	   ' the newly emitted quadrangle is perpendicular To ball moving direction.
	   iQuaternionLookAt(orientation,velocity,_D3DXVECTOR3(0.0,1.0,0.0))
	   ' Orient the emitter To computed orientation
	   iTrailEmitterOrientationSet(TrailEmitter,orientation)
	   ' Emit the trail at a rate of 14 quadrangles per second.
	   iTrailEmitterEmit(TrailEmitter,14.0,1.0)
	EndMethod
	Method _Exit()
	   CMKD=Null
	EndMethod
EndType